using System;
using Microsoft.SharePoint;

namespace RH
{
	public class Alert
	{
		private SPAlert _UserAlert;

		#region properties
		public SPAlert UserAlert
		{
			set { _UserAlert = value; }
		}

		public string WebTitle { get; private set; }

		public string WebUrl { get; private set; }

		public SPAlertFrequency Frequency
		{
			get { return _UserAlert.AlertFrequency; }
		}

		public SPAlertType AlertType
		{
			get { return _UserAlert.AlertType; }
		}

		public bool AlwaysNotify
		{
			get { return _UserAlert.AlwaysNotify; }
		}

		public SPEventType EventType
		{
			get { return _UserAlert.EventType; }
		}

		public string Filter
		{
			get { return _UserAlert.Filter; }
		}

		public string ListName
		{
			get
			{
				return _UserAlert.AlertType == SPAlertType.Custom ? string.Empty : _UserAlert.List.Title;
			}
		}

		public string DefaultDisplayName
		{
			get { return _UserAlert.Title; }
		}

		public string ListDispFormUrl
		{
			get
			{
				if (_UserAlert.AlertType != SPAlertType.Item) return string.Empty;

				int itemID = (_UserAlert.Item != null) ? _UserAlert.ItemID : 0;
				return string.Format("{0}/{1}?ID={2}", WebUrl, _UserAlert.List.Forms[PAGETYPE.PAGE_DISPLAYFORM].Url, itemID);
			}
		}

		public string AlertID
		{
			get { return Convert.ToString(_UserAlert.ID); }
		}

		public string ListUrl
		{
			get
			{
				//if (_UserAlert.AlertTemplate.Name != "OSS.Search")
				if (_UserAlert.AlertType != SPAlertType.Custom)
					return string.Format("{0}/{1}", WebUrl, _UserAlert.ListUrl);

				return WebUrl;
			}
		}

		public Guid? ListID
		{
			get
			{
				//if (_UserAlert.AlertTemplate.Name != "OSS.Search")
				if (_UserAlert.AlertType != SPAlertType.Custom)
				{
					return _UserAlert.ListID;
				}
				return null;
			}
		}
		#endregion

		public Alert(SPAlert alert, SPWeb web)
		{
			_UserAlert = alert;
			WebTitle = web.Title;
			WebUrl = web.ServerRelativeUrl;
		}
	}
}