using System;
using System.Globalization;
using Microsoft.SharePoint;

namespace RH
{
    public class Common
    {
        /// <summary>
        /// returns the current cultureinfo
        /// </summary>
        /// <returns></returns>
        internal static CultureInfo GetCultureInfo(SPWeb web)
        {
            CultureInfo newCultureInfo;
            try
            {
            	var currentUser = web.CurrentUser;
                if (currentUser != null && currentUser.RegionalSettings != null && currentUser.RegionalSettings.LocaleId > 0)
                {
                    uint localeId = currentUser.RegionalSettings.LocaleId;
                    newCultureInfo = new CultureInfo((int)localeId);
                }
                else
                {
                    WriteUlsVerbose("Current User is null (or its regionalSettings). Using web.RegionalSettings");
                    newCultureInfo = new CultureInfo((int)web.RegionalSettings.LocaleId);
                }
            }
            catch (Exception ex)
            {
                WriteUlsError(string.Format("Error in GetCultureInfo for web '{0}': {1}", web.Url, ex));
                newCultureInfo = new CultureInfo(1033);
            }

            return newCultureInfo;
        }

        /// <summary>
        /// will return ex.ToString() or ex.Message depending on the comilation settings
        /// </summary>
        /// <param name="ex"></param>
        /// <returns></returns>
        internal static string GetErrorMessage(Exception ex)
        {
            if (ex != null)
            {
#if DEBUG
                return ex.ToString();
#else
                return ex.Message;
#endif
            }
            return string.Empty;
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Exception)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteUlsError(string text)
        {
            if (MyAlerts.EnableLogging == false) return;

            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Exception, Guid.Empty, "RH.MyAlerts.dll", "RH.MyAlerts", "General", text);
            }
            catch (Exception)
            {
            }
            finally
            {
                TraceProvider.UnregisterTraceProvider();
            }
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Verbose)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteUlsVerbose(string text)
        {
            if (MyAlerts.EnableLogging == false) return;

            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Verbose, Guid.Empty, "RH.MyAlerts.dll", "RH.MyAlerts", "General", text);
            }
            catch (Exception)
            {
            }
            finally
            {
                TraceProvider.UnregisterTraceProvider();
            }
        }
    }
}
