using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;
using System.Reflection;

namespace RH
{
	/// <summary>
	/// Description of the toolpart. Override the GetToolParts method
	/// in your Web Part class to invoke this toolpart. To establish
	/// a reference to the Web Part the user has selected, use the
	/// ParentToolPane.SelectedWebPart property.
	/// </summary>
	public class CustomToolPart : ToolPart
	{
		// declaring variables
		private string _AlertsFromAllSites;
		private string _ShowListColumn;
		private string _ShowFrequencyColumn;
		private string _ShowAlertTypeColumn;
		private string _ShowEventTypeColumn;
		private string _EnableLogging;
		private readonly uint _LocaleId;

		// an event handler for the Init event
		private void CustomToolPartInit(object sender, EventArgs e)
		{
			_AlertsFromAllSites = UniqueID + "AlertsFromAllSites";
			_ShowListColumn = UniqueID + "ShowListColumn";
			_ShowFrequencyColumn = UniqueID + "ShowFrequencyColumn";
			_ShowAlertTypeColumn = UniqueID + "ShowAlertTypeColumn";
			_ShowEventTypeColumn = UniqueID + "ShowEventTypeColumn";
			_EnableLogging = UniqueID + "EnableLogging";
		}

		/// <summary>
		/// Constructor for the class. A great place to set Set
		/// default values for additional base class properties
		/// here.
		/// </summary>
		public CustomToolPart()
		{
			// Set default properties
			Title = "Ren Hzser - My Alerts";
			Init += CustomToolPartInit;

			var cultureInfo = System.Threading.Thread.CurrentThread.CurrentUICulture;
			_LocaleId = (uint)cultureInfo.LCID;
		}

		/// <summary>
		/// Called by the tool pane to apply property changes to the selected Web Part.
		/// </summary>
		public override void ApplyChanges()
		{
			var wp1 = (MyAlerts) ParentToolPane.SelectedWebPart;

			// Send the values to the Web Part.
			wp1.AlertsFromAllSites = (SPEncode.HtmlEncode(Page.Request.Form[_AlertsFromAllSites]) == "on");
			wp1.ShowListColumn = (SPEncode.HtmlEncode(Page.Request.Form[_ShowListColumn]) == "on");
			wp1.ShowFrequencyColumn = (SPEncode.HtmlEncode(Page.Request.Form[_ShowFrequencyColumn]) == "on");
			wp1.ShowAlertTypeColumn = (SPEncode.HtmlEncode(Page.Request.Form[_ShowAlertTypeColumn]) == "on");
			wp1.ShowEventTypeColumn = (SPEncode.HtmlEncode(Page.Request.Form[_ShowEventTypeColumn]) == "on");
			MyAlerts.EnableLogging = (SPEncode.HtmlEncode(Page.Request.Form[_EnableLogging]) == "on");
		}

		/// <summary>
		///	If the ApplyChanges method succeeds, this method is
		/// called by the tool pane to refresh the specified
		/// property values in the toolpart user interface.
		/// </summary>
		public override void SyncChanges()
		{
			// sync with the new property changes here
		}

		/// <summary>
		///	Called by the tool pane if the user discards changes
		/// to the selected Web Part.
		/// </summary>
		public override void CancelChanges()
		{
		}

		/// <summary>
		/// Render this Tool part to the output parameter
		/// specified.
		/// </summary>
		/// <param name="writer">
		/// The HTML writer to write out to
		/// </param>
		protected override void RenderToolPart(HtmlTextWriter writer)
		{
			try
			{
				// Establish a reference to the Web Part.
				var wp1 = (MyAlerts) ParentToolPane.SelectedWebPart;

				//writer.Write(GetString("ToolPart_ShowFromAllSites"));
				CheckBox cb1 = CreateCheckBox(wp1, "AlertsFromAllSites");
				cb1.RenderControl(writer);
				writer.WriteLine("</br></br>");

				CheckBox cb6 = CreateCheckBox(wp1, "EnableLogging");
				cb6.RenderControl(writer);
				writer.Write("</BR></BR>");

				writer.Write(GetString("ToolPart_ShowWhichColumns") + "<BR/>");

				CheckBox cb2 = CreateCheckBox(wp1, "ShowListColumn");
				cb2.RenderControl(writer);
				writer.Write("</BR>");

				CheckBox cb3 = CreateCheckBox(wp1, "ShowFrequencyColumn");
				cb3.RenderControl(writer);
				writer.Write("</BR>");

				CheckBox cb4 = CreateCheckBox(wp1, "ShowAlertTypeColumn");
				cb4.RenderControl(writer);
				writer.Write("</BR>");

				CheckBox cb5 = CreateCheckBox(wp1, "ShowEventTypeColumn");
				cb5.RenderControl(writer);
				writer.Write("</BR>");
			}
			catch (Exception ex)
			{
				writer.Write(string.Format("{0} CreateChildControls:\r\n{1}", GetString("Error_in"), Common.GetErrorMessage(ex)));
			}
		}

		private CheckBox CreateCheckBox(MyAlerts wp1, string property)
		{
			var cb = new CheckBox();

			PropertyInfo[] properties = wp1.GetType().GetProperties();
			foreach (PropertyInfo info in properties)
			{
				if (info.Name != property) continue;

				cb.ID = UniqueID + property;
				cb.Checked = (bool) info.GetValue(wp1, null);
				cb.Text = GetString(string.Format("ToolPart_{0}", property));
			}

			return cb;
		}

		private string GetString(string resourceKey)
		{
			var localized = SPUtility.GetLocalizedString("$Resources:" + resourceKey, "RH.MyAlerts", _LocaleId);
			return localized;
		}
	}
}