using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Resources;
using System.Web;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.WebPartPages;
using System.Security;
using System.Web.UI;

namespace RH
{
	public class MyAlerts : WebPart
	{
		private readonly Version _Version = new Version(1, 1, 0, 0);
		private string _ErrorMessage = string.Empty;

		private readonly ResourceManager _SharePointRm;
		private AlertsGridView _GridView;
		private DataTable _DataTable;
		private ObjectDataSource _ItemDataSource;
		private readonly IList<Alert> _Alerts;
		private SPUser _CurrentUser;
		private CultureInfo _CultureInfo;
		private uint _LocaleId;

		[Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("Show Alerts from all sites")]
		public bool AlertsFromAllSites { get; set; }

		[Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("Show List Column")]
		public bool ShowListColumn { get; set; }

		[Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("Show Frequency Column")]
		public bool ShowFrequencyColumn { get; set; }

		[Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("Show Alert Type Column")]
		public bool ShowAlertTypeColumn { get; set; }

		[Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("Show Event Type Column")]
		public bool ShowEventTypeColumn { get; set; }

		[Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("Enable ULS Logging")]
		public static bool EnableLogging { get; set; }

		public Version Version
		{
			get { return _Version; }
		}

		public override ToolPart[] GetToolParts()
		{
			try
			{
				var toolparts = new ToolPart[3];
				var wptp = new WebPartToolPart();
				var custom = new CustomPropertyToolPart();
				var footer = new CustomPropertyToolPart();
				toolparts[0] = custom;
				toolparts[1] = wptp;
				toolparts[2] = footer;

				// This is the custom ToolPart.
				toolparts[0] = new CustomToolPart();
				toolparts[2] = new CustomToolPartFooter();

				return toolparts;
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("Error in ToolPart: {0}", Common.GetErrorMessage(ex));
			}
			return null;
		}

		public MyAlerts()
		{
			try
			{
				// get the current lcid (e.g. if the user switched languages)
				_CultureInfo = System.Threading.Thread.CurrentThread.CurrentUICulture;
				_LocaleId = (uint)_CultureInfo.LCID;
				_Alerts = new List<Alert>();

				var sharePointIntlAssembly = Assembly.Load("Microsoft.SharePoint.intl, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c");
				_SharePointRm = new ResourceManager("Microsoft.SharePoint", sharePointIntlAssembly);
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("Error in Constructor: {0}", Common.GetErrorMessage(ex));
			}
		}

		protected override void CreateChildControls()
		{
			base.CreateChildControls();
			try
			{
				_CurrentUser = SPContext.Current.Web.CurrentUser;
				_DataTable = new DataTable("DataTableDisplayMyAlerts");
				CreateDataTableColumns();

				_ItemDataSource = new ObjectDataSource("RH.MyAlertsItemSelector, RH.MyAlerts, Version=1.0.0.0, Culture=neutral, PublicKeyToken=b695fcac4a3efe33", "Select");
				_ItemDataSource.ObjectCreating += delegate(object sender, ObjectDataSourceEventArgs e)
				{
					e.ObjectInstance = new MyAlertsItemSelector(_DataTable);
				};
				_ItemDataSource.ID = string.Format("ItemDataSource_{0}", ID);
				Controls.Add(_ItemDataSource);

				HttpRequest req = HttpContext.Current.Request;
				if (req.Form["__CALLBACKID"] == null || req.Form["__CALLBACKPARAM"] == null || !req.Form["__CALLBACKID"].EndsWith("DataTableDisplayMyAlerts"))
				{
					if (!string.IsNullOrEmpty((string)ViewState["FilterExpression"])) _ItemDataSource.FilterExpression = (string)ViewState["FilterExpression"];
				}

				_GridView = new AlertsGridView { ID = string.Format("RH.MyAlerts_Grid_{0}", ID), AutoGenerateColumns = false };
				CreateGridColumns();
				_GridView.DataSourceID = _ItemDataSource.ID;
				_GridView.AllowSorting = true;
				_GridView.AllowFiltering = true;
				_GridView.FilterDataFields = GetFilterField();
				_GridView.FilteredDataSourcePropertyName = "FilterExpression";
				_GridView.FilteredDataSourcePropertyFormat = "{1} LIKE '{0}'";
				_GridView.AllowGrouping = true;
				_GridView.AllowGroupCollapse = true;
				_GridView.GroupField = "WebTitle";
				_GridView.GroupFieldDisplayName = GetSpString("WebObject");
				_GridView.AutoGenerateSelectButton = false;
				_GridView.EnableTheming = false;
				Controls.Add(_GridView);

				GetUserAlertsAndAddDataRows();
				_GridView.DataBind();
			}
			catch (SecurityException ex)
			{
				_ErrorMessage += string.Format("Please change the trust level in your web.config to 'Full':<br/>{0}", ex.Message);
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("{0} CreateChildControls:\r\n{1}", GetString("Error_in"), Common.GetErrorMessage(ex));
			}
		}

		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			try
			{
				ViewState["FilterExpression"] = _ItemDataSource.FilterExpression;

				HideFieldsFromGrid();
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("{0} OnPreRender:\r\n{1}", GetString("Error_in"), Common.GetErrorMessage(ex));
			}
		}

		protected override void Render(HtmlTextWriter writer)
		{
			try
			{
				if (Parent.GetType().FullName == "Microsoft.SharePoint.WebPartPages.WebPartPreview")
				{
					writer.Write("<img src=\"/_layouts/images/RH.MyAlerts/RH.MyAlertsPreview.jpg\" border=\"0\" alt=\"RH.MyAlerts Webpart from Ren Hzser\">");
				}
				else
				{
					EnsureChildControls();

					if (string.IsNullOrEmpty(_ErrorMessage))
					{
						const string js = @"<script type=""text/javascript"">" +
												"function editAlertScript(weburl, alertid, listid, source){" +
												"   var url = weburl + '/_layouts/SubEdit.aspx?Alert=' + alertid;" +
												"   if (listid.length > 0)" +
												"   { url = url + '&List=' + listid; }" +
												"   url = url + '&Source=' + source;" +
												"   window.navigate(url);" +
												"}" +
												"</script>";
						writer.Write(js);

						base.Render(writer);
					}
					else
					{
						var errorControl = new HtmlGenericControl("DIV");
						errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
						errorControl.InnerHtml = "<IMG src=\"/_layouts/images/error.gif\" border=\"0\"/>" + _ErrorMessage;
						errorControl.RenderControl(writer);
					}
				}
			}
			catch (Exception ex)
			{
				var errorControl = new HtmlGenericControl("DIV");
				errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
				errorControl.InnerHtml = "<IMG src=\"/_layouts/images/error.gif\" border=\"0\"/>" + ex;
				writer.Write(ex.ToString());
			}
		}

		public string GetDesignTimeHtml()
		{
			try
			{
				return GetString("DesignTimeHTML") + "<br/><img src=\"/_layouts/images/RH.MyAlerts/RH.MyAlertsPreview.jpg\" border=\"0\" alt=\"RH.MyAlerts Webpart from Ren Hzser\">";
			}
			catch (Exception ex)
			{
				return string.Format("Error in GetDesignTimeHtml: {0}", ex);
			}
		}

		#region Alerts
		private static SPWebApplication[] GetAllWebApplications()
		{
			SPWebApplication[] webApplications = null;
			SPSecurity.RunWithElevatedPrivileges(delegate
			                                     	{
			                                     		SPFarm farm = SPFarm.Local;
			                                     		var service = farm.Services.GetValue<SPWebService>("");

			                                     		webApplications = service.WebApplications.Where(wa => wa.AlertsEnabled && !wa.IsAdministrationWebApplication).ToArray();
			                                     	});
			return webApplications;
		}

		private void GetUserAlerts()
		{
			try
			{
				if (!AlertsFromAllSites)
				{
					// only get alerts from the current web and below
					GetAlertsForUser(SPContext.Current.Web, _CurrentUser);
					return;
				}

				// get alerts from other web applications in our farm
				foreach (SPWebApplication webApplication in GetAllWebApplications())
				{
					foreach (SPSite site in webApplication.Sites)
					{
						try
						{
							site.CatchAccessDeniedException = false;
							SPWeb rootWeb = site.RootWeb;
							SPUser user = GetUser(rootWeb, _CurrentUser);
							if (user == null) continue;

							if (rootWeb.DoesUserHavePermissions(user.LoginName, SPBasePermissions.ViewPages))
							{
								// connect to the site as the current user
								using (var userSite = new SPSite(site.ID, user.UserToken))
								{
									GetAlertsForUser(userSite.RootWeb, user);
								}
							}
						}
// ReSharper disable EmptyGeneralCatchClause
						catch (Exception)
// ReSharper restore EmptyGeneralCatchClause
						{
							// occures e.g. for the SSP if the app pool does not have permissions
						}
						finally
						{
							site.CatchAccessDeniedException = true;
							site.Dispose();
						}
					}
				}
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("{0} GetUserAlerts:\r\n{1}", GetString("Error_in"), Common.GetErrorMessage(ex));
			}
		}

		/// <summary>
		/// get all alerts from the current web, and call method for subwebs
		/// </summary>
		/// <param name="web"></param>
		/// <param name="user"></param>
		private void GetAlertsForUser(SPWeb web, SPUser user)
		{
			// get alerts from this web
			if (user != null)
			{
				foreach (SPAlert alert in web.CurrentUser.Alerts)
				{
					var newAlert = new Alert(alert, web);
					_Alerts.Add(newAlert);
				}
			}

			// get alerts from the subwebs
			foreach (SPWeb subWeb in web.GetSubwebsForCurrentUser())
			{
				try
				{
					GetAlertsForUser(subWeb, user);
				}
				finally
				{
					// dispose the subWeb, so that we free ressources
					subWeb.Dispose();
				}
			}
		}

		/// <summary>
		/// get the user which has the same loginname from the web
		/// </summary>
		/// <param name="web"></param>
		/// <param name="currentUser">get the SPUser from the web, which matches this login name</param>
		/// <returns>SPUser | null</returns>
		private SPUser GetUser(SPWeb web, SPUser currentUser)
		{
			// need to fetch the user again, as it might be another sitecollection. in that case, the userid is different
			SPUserCollection userCollection = web.AllUsers.GetCollection(new[] {currentUser.LoginName});
			if (userCollection.Count == 1)
			{
				return userCollection[0];
			}
			return null;
		}
		#endregion

		#region Grid
		/// <summary>
		/// returns all fields for filtering
		/// </summary>
		/// <returns></returns>
		private void CreateGridColumns()
		{
			try
			{
				#region Menue
				var gridColumn = new BoundField
											{
												DataField = "DefaultDisplayName",
												SortExpression = "DefaultDisplayName",
												HtmlEncode = false,
												HeaderText = GetString("Alert")
											};
				_GridView.Columns.Add(gridColumn);
				gridColumn.Visible = false;

				string sourceUrl = Page.Request.Url.OriginalString;
				var colMenu = new SPMenuField
										{
											HeaderText = GetString("Alert"),
											TextFields = "DefaultDisplayName",
											MenuTemplateId = "AlertListMenu",
											TokenNameAndValueFields = "WEBURL=WebUrl,ALERTID=AlertID,LISTID=ListId,WEBTITLE=WebTitle,LISTNAME=ListName,LISTURL=ListUrl",
											SortExpression = "DefaultDisplayName"
										};

				var alertListMenu = new MenuTemplate { ID = "AlertListMenu" };

				var editAlertMenu = new MenuItemTemplate(GetSpString("AlertEditAlert").Replace("|0 - ", string.Empty), "/_layouts/images/edit.gif")
												{
													ClientOnClickScript = string.Format("editAlertScript('%WEBURL%','%ALERTID%','%LISTID%','{0}');", sourceUrl)
												};
				alertListMenu.Controls.Add(editAlertMenu);

				var sepMenu = new MenuSeparatorTemplate();
				alertListMenu.Controls.Add(sepMenu);

				var webMenu = new MenuItemTemplate(GetString("Menu_GoToWebsite"), "/_layouts/images/HTML16.gif")
										{
											ClientOnClickNavigateUrl = "%WEBURL%"
										};
				alertListMenu.Controls.Add(webMenu);

				var listItemMenu = new MenuItemTemplate(GetString("Menu_GoToList"), "/_layouts/images/itgen.gif")
											  {
												  ClientOnClickNavigateUrl = "%LISTURL%"
											  };
				alertListMenu.Controls.Add(listItemMenu);

				Controls.Add(alertListMenu);
				_GridView.Columns.Add(colMenu);
				#endregion

				CreateBoundField("ListName");
				CreateBoundField("Frequency");
				CreateBoundField("AlertType");
				CreateBoundField("EventType");
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("{0}\r\n", ex);
			}
		}

		private void CreateBoundField(string name)
		{
			var gridColumn = new BoundField { DataField = name, SortExpression = name, HtmlEncode = false };
			switch (name)
			{
				case "WebUrl":
					gridColumn.HeaderText = GetSpString("ObjectTypeWeb") ?? "Web Url";
					break;
				case "ListName":
					gridColumn.HeaderText = GetSpString("ReqAccList") ?? "List Name";
					break;
				case "Frequency":
					gridColumn.HeaderText = GetString("Frequency") ?? "Frequency";
					break;
				case "AlertType":
					gridColumn.HeaderText = GetSpString("TypeColumn") ?? "Alert Type";
					break;
				case "EventType":
					gridColumn.HeaderText = GetSpString("ListItemTypeEvent") ?? "Event Type";
					break;
				default:
					gridColumn.HeaderText = name;
					break;
			}

			_GridView.Columns.Add(gridColumn);
		}

		private static string GetFilterField()
		{
			const string filterString = "WebTitle, Alert, ListName, Frequency, AlertType, EventType";
			return filterString;
		}

		private void GetUserAlertsAndAddDataRows()
		{
			try
			{
				GetUserAlerts();

				foreach (Alert alert in _Alerts)
				{
					// Add a new Row
					DataRow row = _DataTable.NewRow();

					// Add Columns to the new Row
					row["WebTitle"] = alert.WebTitle;
					row["WebUrl"] = alert.WebUrl;
					row["ListName"] = alert.ListName;
					row["ListDispFormUrl"] = alert.ListDispFormUrl;
					row["ListUrl"] = alert.ListUrl;
					row["DefaultDisplayName"] = alert.DefaultDisplayName;
					switch (alert.Frequency)
					{
						case SPAlertFrequency.Daily:
							row["Frequency"] = GetSpString("Daily");
							break;
						case SPAlertFrequency.Immediate:
							row["Frequency"] = GetString("Frequency_Immediate");
							break;
						case SPAlertFrequency.Weekly:
							row["Frequency"] = GetString("Frequency_Weekly");
							break;
					}
					switch (alert.AlertType)
					{
						case SPAlertType.List:
							row["AlertType"] = GetSpString("ReqAccListUp");
							break;
						case SPAlertType.Item:
							row["AlertType"] = GetSpString("AlertItem");
							break;
						case SPAlertType.Custom:
							row["AlertType"] = GetSpString("Custom");
							break;
					}

					switch (alert.EventType)
					{
						case SPEventType.Add:
							row["EventType"] = GetSpString("AlertListAddChanges");
							break;
						case SPEventType.All:
							row["EventType"] = GetSpString("AlertsAllItems");
							break;
						case SPEventType.Delete:
							row["EventType"] = GetSpString("AlertListDeleteChanges");
							break;
						case SPEventType.Discussion:
							row["EventType"] = GetSpString("AlertListDiscussionChanges");
							break;
						case SPEventType.Modify:
							row["EventType"] = GetSpString("AlertListModifyChanges");
							break;
					}
					row["AlwaysNotify"] = Convert.ToString(alert.AlwaysNotify);
					row["Alert"] = alert.DefaultDisplayName;
					row["AlertID"] = Convert.ToString(alert.AlertID);
					row["ListID"] = Convert.ToString(alert.ListID);

					_DataTable.Rows.Add(row);
				}
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("{0} GetUserAlertsAndAddDataRows:\r\n{1}", GetString("Error_in"), Common.GetErrorMessage(ex));
			}
		}

		private void HideFieldsFromGrid()
		{
			// hide fields from the grid
			var al = new ArrayList();
			if (!ShowListColumn) al.Add("ListName");
			if (!ShowFrequencyColumn) al.Add("Frequency");
			if (!ShowAlertTypeColumn) al.Add("AlertType");
			if (!ShowEventTypeColumn) al.Add("EventType");

			foreach (DataControlField field in _GridView.Columns)
			{
				if (al.Contains(field.SortExpression))
				{
					field.Visible = false;
				}
			}
		}
		#endregion

		#region DataTable
		/// <summary>
		/// Creates Columns for _DataTable
		/// </summary>
		private void CreateDataTableColumns()
		{
			try
			{
				// Create Columns
				_DataTable.Columns.Add("Alert");
				_DataTable.Columns.Add("WebTitle");
				_DataTable.Columns.Add("WebUrl");
				_DataTable.Columns.Add("ListName");
				_DataTable.Columns.Add("ListUrl");
				_DataTable.Columns.Add("ListDispFormUrl");
				_DataTable.Columns.Add("DefaultDisplayName");
				_DataTable.Columns.Add("Frequency");
				_DataTable.Columns.Add("AlertType");
				_DataTable.Columns.Add("EventType");
				_DataTable.Columns.Add("AlwaysNotify");
				_DataTable.Columns.Add("AlertID");
				_DataTable.Columns.Add("ListID");
			}
			catch (Exception ex)
			{
				_ErrorMessage += String.Format("{0} CreateDataTable:<br/>{1}", GetString("Error_in"), Common.GetErrorMessage(ex));
			}
		}
		#endregion

		private string GetString(string resourceKey)
		{
			var localized = SPUtility.GetLocalizedString("$Resources:" + resourceKey, "RH.MyAlerts", _LocaleId);
			return localized;
		}

		private string GetSpString(string resourceKey)
		{
			var localized = _SharePointRm.GetString(resourceKey, _CultureInfo);
			return localized;
		}
	}
}